#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.5

$LastChangedRevision: 7607 $
$LastChangedDate:: 2024-01-16 16:23:16 +0100#$

This file is part of SyMSpace.

Copyright (c) 2023, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
import datetime
from symspace.wrapper.symwrapper import SymField
from symspace.wrapper.symwrapper import SymContainer


class EcometricFunctions:

    KEY_ECO = "Ecometrics"
    KEY_CML = "CML"
    KEY_EPD = "EPD"
    KEY_IMPACT = "Impacts"

    @staticmethod
    def set_ecometric_properties(prj: dict, material_dict: dict, container ='Material'):
        """Sets SyMSpace ecometric field/parameter in component based on data in material dictionary.

        Parameters
        ----------
        prj : dict
            SyMSpace component/project
        material_dict : dict
            Material dictionary containing ecometrics data
        container : str
            Name of container in SyMSpace component (default is 'Material')

        Returns
        -------
        Nothing

        """
        try:
            if EcometricFunctions.KEY_ECO in material_dict:
                eco_dict = material_dict.get(EcometricFunctions.KEY_ECO)    # fetch ecometrics dictionary

                eco_container = SymContainer(EcometricFunctions.KEY_ECO)    # create ecometrics container
                prj["Project"][container].addParam(eco_container)           # add ecometrics container to material of project

                '''Loop through all impact assessment methods'''
                for method in eco_dict:
                    method_dict = eco_dict.get(method)          # fetch current impact assessment method
                    method_name = method_dict.get("Method")     # fetch name/description of impact assessment method
                    method_date = method_dict.get("Date")       # fetch date of impact calculation
                    method_source = method_dict.get("Source")   # fetch source of impact calculation
                    impact_dict = method_dict.get(EcometricFunctions.KEY_IMPACT)    # fetch impact dictionary for current method

                    method_container = SymContainer(method)     # create method container
                    eco_container.addParam(method_container)    # add method container to ecometrics container
                    method_container.setDescription(" ".join([method_name, "impact assessment method"]))    # set description of method container
                    method_container.setComment(" ".join(["Data from", method_date, "created by", method_source]))  # set comment for method container

                    '''Loop through all impact categories'''
                    for impact in impact_dict:
                        ''' Create new parameter '''
                        dataset = impact_dict.get(impact)  # fetch dataset for current impact category
                        quantity = EcometricFunctions.get_impact_quantity(dataset.get("unit"))  # convert ecometrics unit
                        param = SymField(dataset.get("variable"))  # create new component field (aka parameter)
                        method_container.addParam(param)  # add parameter to container (has to be added before data is set)
                        param.setDescription(impact)  # set description (equals name of data set)
                        param.setData(dataset.get("value"))  # set value
                        param.setUnit(quantity["unit"])  # set unit
                        param.setTexString(dataset.get("variable").upper())  # set TeX string (same as variable, but uppercase)
                        param.setType("Ecometric")  # mark parameter as ecometric type
                        param.setConstant("True")  # set parameter to constant
                        param.setComment("".join(["!", quantity["name"]]))
            else:
                # Material dictionary does not contain the "Ecometrics" key -> ignore this material and exit
                pass
        except TypeError:
            # Material dictionary is not iterable -> ignore this material and exit
            pass
        except KeyError:
            # Material dictionary does not contain the "Ecometrics" key -> ignore this material and exit
            pass

    @staticmethod
    def get_impact_quantity(unit: str):
        """Separates the information contained in a unit string from an impact quantity.

        Example: "kg CO2-Eq / kg" will be split to "CO2-Eq" as name and "kg / kg" as unit

        Parameters
        ----------
        unit : str
            Full unit string

        Returns
        -------
        quantity : dict
            Dictionary containing name and unit of quantity
        """
        '''Divide full unit string'''
        num, denom = unit.rsplit("/", 1)    # divide string in numerator and denominator
        split = num.split(" ", 1)           # split numerator at first white space

        '''Conquer quantity elements'''
        num_unit = split[0]                 # numerator of quantity unit is leading part of full numerator
        denom_unit = denom                  # denominator of quantity unit is denominator of full unit
        if len(split) > 1:
            name = split[1]                 # quantity name is trailing part of full numerator
        else:
            name = " "                       # no trailing part of full numerator -> no quantity name

        '''Assemble quantity dictionary'''
        quantity = {"name": name, "unit": " / ".join([num_unit.strip(), denom_unit.strip()])}

        return quantity

    @staticmethod
    def read_eco_excel_file(excel_file: str):
        """Reads data from ecometrics Excel file.

        Parameters
        ----------
        excel_file : str
            Microsoft Excel file with ecometrics data created by openLCA

        Returns
        -------
        material_dictionary_CML : dict
            Material dictionary

        """
        import pandas as pd

        book = pd.read_excel(excel_file, sheet_name=None, header=None, engine='openpyxl')

        ''' General information '''
        sheet = book[list(book.keys())[0]]  # fetch information from first sheet "Calculation setup"
        impact_data = {}
        impact_data["product"] = sheet.iloc[2, 2]  # "Product system"
        impact_data["method"] = sheet.iloc[7, 2]   # "Impact method"
        amount = sheet.iloc[6, 2].split()          # "Amount"
        date = sheet.iloc[11, 2]                   # "Date"
        impact_data["date"] = pd.to_datetime(date).date().isoformat() # convert Excel's date format (float number) to YYYY-MM-DD format (string)
        # get first column of sheet
        first_col = sheet.iloc[:, 1]
        if "Source:" in first_col.array:
            '''Source information in Excel file available -> fetch it'''
            impact_data["source"] = sheet.iloc[first_col[first_col == "Source:"].index[0], 2]
        else:
            '''Source information in Excel file not present -> file was generated by openLCA'''
            impact_data["source"] = "LCM GmbH with openLCA and ecoinvent"

        method = EcometricFunctions.get_impact_method(impact_data["method"])
        material_dictionary = {EcometricFunctions.KEY_ECO: {method: {EcometricFunctions.KEY_IMPACT: {}}}}

        print('Ecomectrics data:')
        print('-----------------')
        print("Product system: ", impact_data.get("product"))
        print("Impact method: ", impact_data.get("method"))
        print("Date: ", impact_data.get("date"))
        print("Source:", impact_data.get("source"))
        material_dictionary[EcometricFunctions.KEY_ECO][method]["Method"] = impact_data.get("method")
        material_dictionary[EcometricFunctions.KEY_ECO][method]["Source"] = impact_data.get("source")
        material_dictionary[EcometricFunctions.KEY_ECO][method]["Date"] = impact_data.get("date")

        ''' Impact data '''
        sheet = book[list(book.keys())[2]]  # fetch information from third sheet "Impacts"
        for i in range(2, sheet.shape[0]):
            ''' fetch impact data row by row '''
            impact_data["name"] = sheet.iloc[i, 2]  # column "Impact category"
            impact_data["unit"] = " / ".join([sheet.iloc[i, 3], amount[1]])   # column "Reference unit" per unit of reference amount
            impact_data["value"] = sheet.iloc[i, 4] / float(amount[0])        # column "Result" divided by reference amount
            print("Impact: ", impact_data.get("name"), " ", impact_data.get("value"), " ", impact_data.get("unit"))

            ''' enter data in material dictionary '''
            material_dictionary = EcometricFunctions.add_impact_data(material_dictionary, impact_data, method)

        return material_dictionary

    @staticmethod
    def add_impact_data(material_dictionary: dict, impact_data: dict, method: str):
        """Enters impact data from specified impact assessment method in material dictionary

        Parameters
        ----------
        material_dictionary : dict
            Original material dictionary
        impact_data : dict
            Ecometrics impact assessment data
        method : str
            Ecometrics impact assessment method

        Returns
        -------
        material_dictionary : dict
            Modified material dictionary

        """
        if method == EcometricFunctions.KEY_CML:
            material_dictionary = EcometricFunctions.add_impact_data_cml(material_dictionary, impact_data)
        elif method == EcometricFunctions.KEY_EPD:
            material_dictionary = EcometricFunctions.add_impact_data_epd(material_dictionary, impact_data)
        else:
            print("Warning: Impact assessment method ", method, "not supported!")
        return material_dictionary

    @staticmethod
    def add_impact_data_cml(material_dictionary: dict, impact_data: dict):
        """Enters impact data from CML impact assessment method in material dictionary.

        Parameters
        ----------
        material_dictionary : dict
            Original material dictionary
        impact_data : dict
            Ecometrics impact assessment data

        Returns
        -------
        material_dictionary : dict
            Modified material dictionary

        """
        if "acidification - acidification" in impact_data.get("name"):
            material_dictionary[EcometricFunctions.KEY_ECO][EcometricFunctions.KEY_CML][EcometricFunctions.KEY_IMPACT]["Acidification potential"] = {"variable": "ap",
                                                                                   "value": impact_data.get("value"),
                                                                                   "unit": impact_data.get("unit")}
        elif "climate change - global warming potential" in impact_data.get("name"):
            material_dictionary[EcometricFunctions.KEY_ECO][EcometricFunctions.KEY_CML][EcometricFunctions.KEY_IMPACT]["Global warming potential"] = {"variable": "gwp",
                                                               "value": impact_data.get("value"),
                                                               "unit": impact_data.get("unit")}
        elif "ecotoxicity: freshwater" in impact_data.get("name"):
            material_dictionary[EcometricFunctions.KEY_ECO][EcometricFunctions.KEY_CML][EcometricFunctions.KEY_IMPACT]["Freshwater aquatic ecotoxicity"] = {"variable": "fae",
                                                                     "value": impact_data.get("value"),
                                                                     "unit": impact_data.get("unit")}
        elif "ecotoxicity: marine" in impact_data.get("name"):
            material_dictionary[EcometricFunctions.KEY_ECO][EcometricFunctions.KEY_CML][EcometricFunctions.KEY_IMPACT]["Marine aquatic ecotoxicity"] = {"variable": "mae",
                                                                 "value": impact_data.get("value"),
                                                                 "unit": impact_data.get("unit")}
        elif "ecotoxicity: terrestrial" in impact_data.get("name"):
            material_dictionary[EcometricFunctions.KEY_ECO][EcometricFunctions.KEY_CML][EcometricFunctions.KEY_IMPACT]["Terrestrial aquatic ecotoxicity"] = {"variable": "tae",
                                                                      "value": impact_data.get("value"),
                                                                      "unit": impact_data.get("unit")}
        elif "energy resources: non-renewable - abiotic depletion potential" in impact_data.get("name"):
            material_dictionary[EcometricFunctions.KEY_ECO][EcometricFunctions.KEY_CML]["Impacts"]["Abiotic depletion potential - fossil fuel"] = {"variable": "adpf",
                                                                                "value": impact_data.get("value"),
                                                                                "unit": impact_data.get("unit")}
        elif "eutrophication - eutrophication" in impact_data.get("name"):
            material_dictionary[EcometricFunctions.KEY_ECO][EcometricFunctions.KEY_CML][EcometricFunctions.KEY_IMPACT]["Eutrophication potential"] = {"variable": "np",
                                                               "value": impact_data.get("value"),
                                                               "unit": impact_data.get("unit")}
        elif "human toxicity - human toxicity" in impact_data.get("name"):
            material_dictionary[EcometricFunctions.KEY_ECO][EcometricFunctions.KEY_CML][EcometricFunctions.KEY_IMPACT]["Human toxicity"] = {"variable": "htp",
                                                     "value": impact_data.get("value"),
                                                     "unit": impact_data.get("unit")}
        elif "material resources: metals/minerals - abiotic depletion potential" in impact_data.get("name"):
            material_dictionary[EcometricFunctions.KEY_ECO][EcometricFunctions.KEY_CML][EcometricFunctions.KEY_IMPACT]["Abiotic depletion potential - elements"] = {"variable": "adpe",
                                                                             "value": impact_data.get("value"),
                                                                             "unit": impact_data.get("unit")}
        elif "ozone depletion - ozone layer depletion" in impact_data.get("name"):
            material_dictionary[EcometricFunctions.KEY_ECO][EcometricFunctions.KEY_CML][EcometricFunctions.KEY_IMPACT]["Ozone depletion potential"] = {"variable": "odp",
                                                                "value": impact_data.get("value"),
                                                                "unit": impact_data.get("unit")}
        elif "photochemical oxidant formation - photochemical oxidation" in impact_data.get("name"):
            material_dictionary[EcometricFunctions.KEY_ECO][EcometricFunctions.KEY_CML][EcometricFunctions.KEY_IMPACT]["Photochemical ozone creation potential"] = {"variable": "pocp",
                                                                             "value": impact_data.get("value"),
                                                                             "unit": impact_data.get("unit")}

        return material_dictionary

    @staticmethod
    def add_impact_data_epd(material_dictionary: dict, impact_data: dict):
        """Enters impact data from EPD - Environmental Product Declaration (impact assessment method according to EN15804) in material dictionary.

        Parameters
        ----------
        material_dictionary : dict
            Original material dictionary
        impact_data : dict
            Ecometrics impact assessment data

        Returns
        -------
        material_dictionary : dict
            Modified material dictionary

        """
        if "AP" in impact_data.get("name"):
            material_dictionary[EcometricFunctions.KEY_ECO][EcometricFunctions.KEY_EPD][EcometricFunctions.KEY_IMPACT]["Acidification potential"] = {"variable": "ap",
                                                                                   "value": impact_data.get("value"),
                                                                                   "unit": impact_data.get("unit")}
        elif "GWP" and "total" in impact_data.get("name"):
            material_dictionary[EcometricFunctions.KEY_ECO][EcometricFunctions.KEY_EPD][EcometricFunctions.KEY_IMPACT]["Global warming potential"] = {"variable": "gwp",
                                                               "value": impact_data.get("value"),
                                                               "unit": impact_data.get("unit")}
        elif "EP" and "freshwater" in impact_data.get("name"):
            material_dictionary[EcometricFunctions.KEY_ECO][EcometricFunctions.KEY_EPD][EcometricFunctions.KEY_IMPACT]["Freshwater aquatic ecotoxicity"] = {"variable": "fae",
                                                                     "value": impact_data.get("value"),
                                                                     "unit": impact_data.get("unit")}
        elif "EP " and "marine" in impact_data.get("name"):
            material_dictionary[EcometricFunctions.KEY_ECO][EcometricFunctions.KEY_EPD][EcometricFunctions.KEY_IMPACT]["Marine aquatic ecotoxicity"] = {"variable": "mae",
                                                                 "value": impact_data.get("value"),
                                                                 "unit": impact_data.get("unit")}
        elif "EP" and "terrestrial" in impact_data.get("name"):
            material_dictionary[EcometricFunctions.KEY_ECO][EcometricFunctions.KEY_EPD][EcometricFunctions.KEY_IMPACT]["Terrestrial aquatic ecotoxicity"] = {"variable": "tae",
                                                                      "value": impact_data.get("value"),
                                                                      "unit": impact_data.get("unit")}
        elif "ADP" and "fossil" in impact_data.get("name"):
            material_dictionary[EcometricFunctions.KEY_ECO][EcometricFunctions.KEY_EPD]["Impacts"]["Abiotic depletion potential - fossil resources"] = {"variable": "adpf",
                                                                                "value": impact_data.get("value"),
                                                                                "unit": impact_data.get("unit")}
        elif "ADP" and "mineral" in impact_data.get("name"):
            material_dictionary[EcometricFunctions.KEY_ECO][EcometricFunctions.KEY_EPD][EcometricFunctions.KEY_IMPACT]["Abiotic depletion potential - elements"] = {"variable": "adpe",
                                                                             "value": impact_data.get("value"),
                                                                             "unit": impact_data.get("unit")}
        elif "ODP" in impact_data.get("name"):
            material_dictionary[EcometricFunctions.KEY_ECO][EcometricFunctions.KEY_EPD][EcometricFunctions.KEY_IMPACT]["Ozone depletion potential"] = {"variable": "odp",
                                                                "value": impact_data.get("value"),
                                                                "unit": impact_data.get("unit")}
        elif "POCP" in impact_data.get("name"):
            material_dictionary[EcometricFunctions.KEY_ECO][EcometricFunctions.KEY_EPD][EcometricFunctions.KEY_IMPACT]["Photochemical ozone creation potential"] = {"variable": "pocp",
                                                                             "value": impact_data.get("value"),
                                                                             "unit": impact_data.get("unit")}
        elif "WDP" in impact_data.get("name"):
            material_dictionary[EcometricFunctions.KEY_ECO][EcometricFunctions.KEY_EPD][EcometricFunctions.KEY_IMPACT]["Water deprivation potential"] = {"variable": "wdp",
                                                     "value": impact_data.get("value"),
                                                     "unit": impact_data.get("unit")}

        return material_dictionary

    @staticmethod
    def get_impact_method(method: str):
        """Creates a key/abbreviation of the specified impact assessment method.

        Parameters
        ----------
        method : str
            Full name of impact assessment method

        Returns
        -------
        key : str
            Impact assessment method key, i.e. an abbreviation of the method. The return string is 'unknown', if method is not supported.
        """
        key = "unknown"

        if "CML" in method.upper():
            key = EcometricFunctions.KEY_CML
        elif "EPD" in method.upper():
            key = EcometricFunctions.KEY_EPD

        return key


    @staticmethod
    def extract_eco_data():
        # ToDo: extract data from openLCA directly (future implementation)
        pass
