#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 7607 $
$LastChangedDate:: 2024-01-16 16:23:16 +0100#$

This file is part of SyMSpace.

Copyright (c) 2017, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
import os
import shutil
import json
import glob
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.font_manager import FontProperties

from symspace.wrapper.symwrapper import SymProject

import componentspace.osutils.fileutils as fu
from componentspace.material_utils.material_functions import GeneralMaterialFunctions
from componentspace.material_utils.batch_material import MaterialBatchComponent
from componentspace.material_utils.ecometric_functions import EcometricFunctions


class MaterialFerromagnetic(MaterialBatchComponent):
    """

    """

    def _createSingleCompfileFromTemplate(self, comp_file: str, material: dict):
        path_material = os.path.dirname(comp_file)
        component_name = material['component_name']

        '''Copy rst'''
        fu.copyFileAndReplace(os.path.join(self.path, 'index.rst'),
                              os.path.join(path_material, 'index.rst'),
                              [('__ComponentTitle__',
                                component_name + '\n' + len(component_name) * '='),
                               ('__Comment__', material['comment']),
                               ('__MaterialName__', material['name']),
                               ('__Manufacturer__', material['manufacturer']),
                               ('__BHCurve__', (component_name + '_bhcurve_plot.png')),
                               ('__PDF__', material['pdf_file']),
                               ('__EXCEL__', material['excel_file'])])

        fu.copyTree(os.path.join(self.path, 'rst'), os.path.join(path_material, 'rst'))

        '''Copy pdf'''
        pdf_file = os.path.join(self.path, 'resources', material['pdf_file'])
        if os.path.isfile(pdf_file):
            shutil.copy(pdf_file, os.path.join(path_material, 'rst', material['pdf_file']))

        '''Copy excel file'''
        excel_file = os.path.join(self.path, 'resources', material['excel_file'])
        if os.path.isfile(excel_file):
            shutil.copy(excel_file, os.path.join(path_material, 'rst', material['excel_file']))

        '''Copy png file'''
        bhcurve_png_file = os.path.join(path_material, 'rst', (component_name + '_bhcurve_plot.png'))
        self.plot_bhcurve(material, bhcurve_png_file)

        '''Modify material file'''
        prj = SymProject.load(comp_file)
        prj['Project.Material.Type'].setData('Ferromagnetic material')
        prj['Project.Material.Name'].setData(component_name)

        GeneralMaterialFunctions.set_filter(prj, material)
        GeneralMaterialFunctions.set_info(prj, material)
        GeneralMaterialFunctions.set_general_properties(prj, material)
        GeneralMaterialFunctions.set_electromagnetic_properties(prj, material)
        GeneralMaterialFunctions.set_mechanic_properties(prj, material)
        GeneralMaterialFunctions.set_thermal_properties(prj, material)
        EcometricFunctions.set_ecometric_properties(prj, material)

        prj.save()

    @staticmethod
    def plot_bhcurve(material_dictionary, png_file):
        mu_0 = 4 * np.pi * 1e-7

        fontP = FontProperties()
        fontP.set_size('small')
        fig2 = plt.figure()

        # fig2.figsize(15,25)
        fig2.set_figheight(20)
        fig2.set_figwidth(15)

        ax1 = fig2.add_subplot(311)  # ,figsize=(15,15))
        ax1.grid(True)
        ax1.set_title(r'$Curve \ for \ FE \ simulation \ vs. \ measurement$')  # "Curve for FE simulation")
        ax1.set_xlabel(r"$H [A/m]$")
        ax1.set_ylabel(r"$B [T]$")

        max_h = material_dictionary['H_original'][0][-1]
        max_b = material_dictionary['B_original'][0][-1]
        ax1.set_xlim(-100, max_h * 1.1)
        ax1.set_ylim(0, max_b * 1.1)

        plt.plot(material_dictionary['H_original'][0], material_dictionary['B_original'][0], "ro")
        plt.plot(material_dictionary['H_extrapolated'][0], material_dictionary['B_extrapolated'][0], "b-")
        # plt.plot(material_dictionary['H_corrected'][0], material_dictionary['B_corrected'][0], "g-")

        box = ax1.get_position()
        ax1.set_position([box.x0, box.y0, box.width * 0.8, box.height])
        ax1.legend(prop=fontP, loc='center left', bbox_to_anchor=(1, 0.5))

        ax2 = fig2.add_subplot(312)
        ax2.grid(True)
        ax2.set_title(r'$\mu_r-Curve$')
        ax2.set_xlabel(r"$B [T]$")
        ax2.set_ylabel(r"$\mu_{r} [1]$")

        mu_r = np.array(material_dictionary['B_corrected'][0][1:]) / np.array(
            material_dictionary['H_corrected'][0][1:]) / mu_0
        mu_r = np.append(0, mu_r)

        ax2.set_xlim(0, max_b * 1.1)
        ax2.set_ylim(0, max(mu_r) * 1.1)

        plt.plot(material_dictionary['B_corrected'][0], mu_r)

        box = ax2.get_position()
        ax2.set_position([box.x0, box.y0, box.width * 0.8, box.height])
        ax2.legend(prop=fontP, loc='center left', bbox_to_anchor=(1, 0.5))

        ax3 = fig2.add_subplot(313)
        ax3.grid(True)
        ax3.set_title(r'$BH-Curve \ for \ FE \ simulation$')  # "Curve for FE simulation")
        ax3.set_xlabel(r"$H [A/m]$")
        ax3.set_ylabel(r"$B [T]$")

        max_h = material_dictionary['H_extrapolated'][0][-1]
        max_b = material_dictionary['B_extrapolated'][0][-1]

        ax3.set_xlim(-4000, max_h * 1.1)
        ax3.set_ylim(0, max_b * 1.1)

        # plt.plot(material_dictionary['H_original'][0], material_dictionary['B_original'][0], "ro")
        plt.plot(material_dictionary['H_extrapolated'][0], material_dictionary['B_extrapolated'][0], "b-")
        # plt.plot(material_dictionary['H_corrected'][0], material_dictionary['B_corrected'][0], "g-")

        box = ax3.get_position()
        ax3.set_position([box.x0, box.y0, box.width * 0.8, box.height])
        ax3.legend(prop=fontP, loc='center left', bbox_to_anchor=(1, 0.5))

        # sio.savemat('\Mat_' + name + '_SimCurves.mat', mdict={'vals': initial_curveMod})

        fig2.tight_layout()
        # fig2.subplots_adjust(hspace=1.0, wspace=1.0)

        fig2.show()
        fig2.savefig(png_file)

    def extractMaterialData(self):
        # Todo: this needs to be cleaned up or re-implemented,
        #  so the functionality is split in smaller methods/functions.
        #  Many code lines are duplicates in MaterialFerromagetic, MaterialVoest and MaterialCogentPower
        from componentspace.material_utils.losscoefficient_functions import MaterialFunctions
        from componentspace.material_utils.bhcurve_functions import ExtendBHCurve, BHCurve_correction

        tasks = dict(only_correct=False, only_extend=True,
                     correct_extend=False, extend_correct=False, extend_correct_part=False)

        file_model = os.path.join(self.path, self.name + '.mop')
        if os.path.isfile(file_model):
            self.sym_mdl = SymProject.load(file_model)
        else:
            raise Exception('Model {} file does not exist!'.format(file_model))

        lst_val = []

        folder_resources = str(os.path.join(self.path, 'resources'))

        ### Test with single file ####
        # all_materials_dictionary = {}
        # pdf_file = r'DB_isovac_330-50A_HC_160418.pdf'  # r'DB_isovac_HP_600-50A_E_290318.pdf'  #
        # excel_file = r'simulation_data_isovac_330-50A_HC.xlsx'  # r'Simulation_data_isovac_HP_600-50A.xlsx'  #
        #
        # full_pdf_path = os.path.join(folder, pdf_file)
        # full_excel_path = os.path.join(folder, excel_file)
        #
        # ## read variables from the pdf-file (grade, Rp02, Rm, rho, ...)
        # pdf_dictionary = read_pdf_file(full_pdf_path, export=True)
        # pdf_dictionary['pdf_file'] = pdf_file
        #
        # ## read information from excel-file (J-, H-, P-values)
        # xls_dictionary = read_excel_file(full_excel_path)
        # xls_dictionary['excel_file'] = excel_file
        #
        # ## Merge contents of dict2 in dict1
        # xls_dictionary.update(pdf_dictionary)
        #
        # all_materials_dictionary['M330-50A'] = xls_dictionary
        ###

        '''All Files'''
        all_materials_dictionary = {}

        os.chdir(folder_resources)
        # print(os.getcwd())
        for i, filename in enumerate(glob.glob(folder_resources + "/*.json")):
            print('**** {} ****'.format(filename))

            with open(filename, "r") as json_file:
                data = json.load(json_file)

            all_materials_dictionary[filename] = data

            if 'excel_file' in data:
                excel_file = data['excel_file']
                print('excel_file: ', excel_file)
                xls_data = self.read_excel_file(os.path.join(folder_resources, excel_file))
                all_materials_dictionary[filename].update(xls_data)

            else:
                print('No excel-file with data for the BH-Curve was given.')

            if 'eco_excel_file' in data:
                eco_excel_file = data['eco_excel_file']
                print('eco_excel_file: ', eco_excel_file)
                eco_xls_data = EcometricFunctions.read_eco_excel_file(os.path.join(folder_resources, eco_excel_file))
                all_materials_dictionary[filename].update(eco_xls_data)
            else:
                print('No eco-excel-file with ecometrics data was given.')

        for key in all_materials_dictionary.keys():
            material_dictionary = all_materials_dictionary[key]

            '''
            Calculation of the loss coefficients kefe and khfe
            !!! We need ONE value which is reasonable for most of the frequencies !!!
            !!! Additionally a function with respect to the frequency is useful   !!!
            !!! in the future                                                     !!!
            '''
            material_mp = MaterialFunctions.calc_loss_coefficients_MP(material_dictionary)

            material_dictionary['khfe_F'] = material_mp['khfe_F']
            material_dictionary['kefe_F'] = material_mp['kefe_F']
            material_dictionary['khfe'] = {"variable": "khfe", "value": material_mp['khfe'], "unit": "W.kg-1"}
            material_dictionary['kefe'] = {"variable": "kefe", "value": material_mp['kefe'], "unit": "W.kg-1"}

            '''
            Different versions of bh-curve extension and correction. The following versions where implemented:
            
            H_extrapolated, B_extrapolated = ExtendBHCurve.extend_bhcurve_arctan(material_dictionary)
            H_extrapolated, B_extrapolated = ExtendBHCurve.interpolate_bhcurve_arctan(material_dictionary)
            H_extrapolated, B_extrapolated = ExtendBHCurve.interpolate_bhcurve_mur(material_dictionary)
            H_extrapolated, B_extrapolated = ExtendBHCurve.extend_bhcurve(material_dictionary, type[0], False)

            !!! BHCurve_correction.correct_bhcurve - which is currently used should !!!
            !!! be replaced by a function implementing the Fröhlich-Kennelly extension !!!

            H_extrapolated, B_extrapolated = ExtendBHCurve.extend_bhcurve_froehlich_kennelly\
                (material_dictionary, material_dictionary['H_original'][0], material_dictionary['B_original'][0], False)
            '''

            mu_0 = 4 * np.pi * 1e-7

            if 'H Values' in material_dictionary:
                # print('Mfs: ', material_dictionary['H Values']['value'][0])
                H = material_dictionary['H Values']['value'][0]
                B = [a + b for a, b in
                     zip([j / 1000.0 for j in material_dictionary['J Values']['value'][0]],
                         [h * mu_0 for h in material_dictionary['H Values']['value'][0]])]
            else:
                # print('Mfs: ', material_dictionary['Magnetic field strength']['value'][0])
                H = material_dictionary['Magnetic field strength']['value'][0]
                B = [a + b for a, b in
                     zip([j / 1000.0 for j in material_dictionary['Magnetic polarization']['value'][0]],
                         [h * mu_0 for h in material_dictionary['Magnetic field strength']['value'][0]])]
            if H[0] > 0.0:
                H.insert(0, 0.0)
                B.insert(0, 0.0)

            # print('H: ', H)
            # print('B: ', B)
            material_dictionary['H_original'] = [H]
            material_dictionary['B_original'] = [B]

            '''Start: only correct'''
            if tasks['only_correct']:
                H_corrected, B_corrected = BHCurve_correction.correct_bhcurve \
                    (material_dictionary, material_dictionary['H_original'][0], material_dictionary['B_original'][0])
                material_dictionary['B_corrected'] = [B_corrected]
                material_dictionary['H_corrected'] = [H_corrected]
            '''End: only correct'''

            '''Start: only extend'''
            if tasks['only_extend']:
                # type = ['arctan', 'exp', 'logistic']
                # H_extrapolated, B_extrapolated = ExtendBHCurve.extend_bhcurve \
                #     (material_dictionary, type[0], material_dictionary['H_original'][0],
                #      material_dictionary['B_original'][0])
                H_extrapolated, B_extrapolated = ExtendBHCurve.extend_bhcurve_froehlich_kennelly(material_dictionary,
                                                                                                 material_dictionary[
                                                                                                     'H_original'][0],
                                                                                                 material_dictionary[
                                                                                                     'B_original'][0],
                                                                                                 False)
                material_dictionary['B_extrapolated'] = [B_extrapolated.tolist()]
                material_dictionary['H_extrapolated'] = [H_extrapolated.tolist()]
                material_dictionary['B_corrected'] = [B_extrapolated.tolist()]
                material_dictionary['H_corrected'] = [H_extrapolated.tolist()]
                # print('H_extrapolated: ', H_extrapolated)
                # print('B_extrapolated: ', B_extrapolated)
                material_dictionary['B'] = [B_extrapolated.tolist()]
                material_dictionary['H'] = [H_extrapolated.tolist()]
            '''End: only extend'''

            '''Start: first correct, then extend'''
            if tasks['correct_extend']:
                H_corrected, B_corrected = BHCurve_correction.correct_bhcurve(material_dictionary,
                                                                              material_dictionary['H_original'][0],
                                                                              material_dictionary['B_original'][0],
                                                                              False)
                material_dictionary['B_corrected'] = [B_corrected]
                material_dictionary['H_corrected'] = [H_corrected]

                # type = ['arctan', 'exp', 'logistic']
                # H_extrapolated, B_extrapolated = ExtendBHCurve.extend_bhcurve\
                #     (material_dictionary, type[0], material_dictionary['H_corrected'][0],
                #      material_dictionary['B_corrected'][0])
                H_extrapolated, B_extrapolated = ExtendBHCurve.extend_bhcurve_froehlich_kennelly \
                    (material_dictionary, material_dictionary['H_corrected'][0], material_dictionary['B_corrected'][0],
                     False)
                material_dictionary['B_extrapolated'] = [B_extrapolated]
                material_dictionary['H_extrapolated'] = [H_extrapolated]
                # print('H_extrapolated: ', H_extrapolated)
                # print('B_extrapolated: ', B_extrapolated)
                material_dictionary['B'] = [B_extrapolated]
                material_dictionary['H'] = [H_extrapolated]
            '''End: first correct, then extend'''

            '''Start: first extend, then correct'''
            if tasks['extend_correct']:
                # type = ['arctan', 'exp', 'logistic']
                # H_extrapolated, B_extrapolated = ExtendBHCurve.extend_bhcurve\
                #     (material_dictionary, type[0], material_dictionary['H_original'][0], material_dictionary['B_original'][0])
                H_extrapolated, B_extrapolated = ExtendBHCurve.extend_bhcurve_froehlich_kennelly \
                    (material_dictionary, material_dictionary['H_original'][0], material_dictionary['B_original'][0],
                     False)
                material_dictionary['B_extrapolated'] = [B_extrapolated]
                material_dictionary['H_extrapolated'] = [H_extrapolated]
                # print('H_extrapolated: ', H_extrapolated)
                # print('B_extrapolated: ', B_extrapolated)

                H_corrected, B_corrected = BHCurve_correction.correct_bhcurve \
                    (material_dictionary, material_dictionary['H_extrapolated'][0],
                     material_dictionary['B_extrapolated'][0], False)
                material_dictionary['B_corrected'] = [B_corrected]
                material_dictionary['H_corrected'] = [H_corrected]
                material_dictionary['B'] = [B_corrected]
                material_dictionary['H'] = [B_corrected]
            '''End: first extend, then correct'''

            '''Start: first extend, then correct part'''
            if tasks['extend_correct_part']:
                # type = ['arctan', 'exp', 'logistic']
                # H_extrapolated, B_extrapolated = ExtendBHCurve.extend_bhcurve\
                #     (material_dictionary, type[0], material_dictionary['H_original'][0],
                #      material_dictionary['B_original'][0])
                H_extrapolated, B_extrapolated = ExtendBHCurve.extend_bhcurve_froehlich_kennelly \
                    (material_dictionary, material_dictionary['H_original'][0], material_dictionary['B_original'][0],
                     False)
                material_dictionary['B_extrapolated'] = [B_extrapolated]
                material_dictionary['H_extrapolated'] = [H_extrapolated]
                # print('H_extrapolated: ', H_extrapolated)
                # print('B_extrapolated: ', B_extrapolated)

                B_Test = [material_dictionary['B_extrapolated'][0][0]]
                H_Test = [material_dictionary['H_extrapolated'][0][0]]
                B_Rest = []
                H_Rest = []
                for i in range(len(material_dictionary['H_extrapolated'][0]) - 1):
                    if (material_dictionary['B_extrapolated'][0][i + 1] - material_dictionary['B_extrapolated'][0][i]) \
                            / (
                            material_dictionary['H_extrapolated'][0][i + 1] - material_dictionary['H_extrapolated'][0][
                        i]) \
                            / mu_0 > 1.5:
                        B_Test.append(material_dictionary['B_extrapolated'][0][i + 1])
                        H_Test.append(material_dictionary['H_extrapolated'][0][i + 1])
                    else:
                        B_Rest.append(material_dictionary['B_extrapolated'][0][i + 1])
                        H_Rest.append(material_dictionary['H_extrapolated'][0][i + 1])

                H_corrected, B_corrected = BHCurve_correction.correct_bhcurve(material_dictionary, H_Test, B_Test,
                                                                              False)
                material_dictionary['B_corrected'] = [B_corrected].append(B_Rest)
                material_dictionary['H_corrected'] = [H_corrected].append(H_Rest)
            '''End: first extend, then correct part'''

            lst_val.append(material_dictionary)

            # select = ('Grade', 'Frequency [Hz]', 'Magnetic polarization [mT]', 'Magnetic field strength [A/m]',
            #           'Specific total loss [W/kg]', 'Relative permeability ')
            #

            #     row = list(ws.get_squared_range(ws.min_column, k, ws.max_column, k))[0]
            #     if row[title.index('Grade')].value is None:
            #         continue
            #
            #     val = {}
            #     for name in select:
            #         cell = row[title.index(name)].value
            #         # print(name + ' :: ' + str(cell))
            #         if name == 'Grade':
            #             val[name] = cell
            #             val['FileName'] = cell.replace('/', '_').replace(' ', '')
            #         elif isinstance(cell, str) and '~' in cell:
            #             rng = cell.split('~')
            #             val[name] = (float(rng[0].strip()) + float(rng[1].strip())) / 2
            #         elif isinstance(cell, str) and '-' in cell:
            #             rng = cell.split('-')
            #             try:
            #                 val[name] = (float(rng[0].strip()) + float(rng[1].strip())) / 2
            #             except:
            #                 val[name] = None
            #         else:
            #             val[name] = cell
            #
            #     lst_val.append(val)

            # print(material_dictionary)
            # Todo: instead of the 'src' folder the 'resources' or a specific folder should be used,
            # so it is the same for all "Batch" processed materials!
            json_folder = str(os.path.join(self.path, 'src'))
            with open(str(os.path.join(json_folder, material_dictionary['component_name'] + ".json")), "w") as f:
                json.dump(material_dictionary, f, indent=4, sort_keys=False)

    @staticmethod
    def read_excel_file(excel_file):
        """ Read Excel data """
        import pandas as pd

        book = pd.read_excel(excel_file, sheet_name=None, header=None)
        sheet = book[list(book.keys())[0]]

        print("sheet.iloc[1, 0]", sheet.iloc[1, 0])
        if 'Grade' == sheet.iloc[1, 0]:
            material_dictionary = MaterialFerromagnetic.read_excel_10_grade(sheet)
        elif 'Data' == sheet.iloc[1, 0]:
            material_dictionary = MaterialFerromagnetic.read_excel_10_bhcurve(sheet)
        else:
            raise Exception("Format of the Excel-file doesn't meet standardized requirements.")

        # if book has second sheet with mechanic data
        print("book keys", book.keys())
        if len(book.keys()) > 1:
            mech_sheet = book[list(book.keys())[1]]
            if 'Mechanic Data' == mech_sheet.iloc[1, 0]:
                material_dictionary.update(MaterialFerromagnetic.read_excel_stressstrain(mech_sheet))
            else:
                raise Exception("Format of the Excel-file doesn't meet standardized requirements.")

        return material_dictionary

    @staticmethod
    def read_excel_10_grade(sheet):
        """ Excel-file where the data for BH-Curve and losses are in the same tabular.
        In this case "Grade" has to be in the cell (1,0) of the excel sheet. """
        material_dictionary = {}
        mu_0 = 4 * np.pi * 1e-7

        print('Frequency [Hz] == ', sheet.iloc[1, 1])
        frequency = []
        indices_frequency = []
        if 'Frequency [Hz]' == sheet.iloc[1, 1]:
            temp_list = [sheet.iloc[i, 1] for i in range(2, sheet.shape[0])]
            # temp_list = list(filter(None, temp_list))
            # frequency = np.array([float(i) for i in temp_list])

            counter = -1
            for item in temp_list:
                counter += 1
                if item is not None and item == item: # check if item is not None and not NaN
                    frequency.append(float(item))
                    indices_frequency.append(counter)

            indices_frequency.append(sheet.shape[0] - 2)  # work around since there is no frequency value in the last row
            material_dictionary['Frequency'] = {'variable': 'F', 'value': frequency, 'unit': 'Hz'}
        else:
            print('No Frequency defined!')

        print('Magnetic polarization [mT] == ', sheet.iloc[1, 2])
        polarization = []
        if 'Magnetic polarization [mT]' == sheet.iloc[1, 2]:
            for i in range(len(indices_frequency) - 1):
                temp_list = [sheet.iloc[i, 2] for i in
                             range(2 + indices_frequency[i], 2 + indices_frequency[i + 1])]
                temp_list = [float(value) for value in temp_list if value != 0.0 and value != "" and value == value]
                temp_list.insert(0, 0.0)
                polarization.append(temp_list)

            material_dictionary['Magnetic polarization'] = {'variable': 'J', 'value': polarization, 'unit': 'mT'}
        else:
            print('No Magnetic polarization defined!')

        print('Magnetic field strength [A/m] == ', sheet.iloc[1, 3])
        field_strength = []
        if 'Magnetic field strength [A/m]' == sheet.iloc[1, 3]:
            for i in range(len(indices_frequency) - 1):
                temp_list = [sheet.iloc[i, 3] for i in
                             range(2 + indices_frequency[i], 2 + indices_frequency[i + 1])]
                temp_list = [float(value) for value in temp_list if value != 0.0 and value != "" and value == value]
                temp_list.insert(0, 0.0)
                field_strength.append(temp_list)

            material_dictionary['Magnetic field strength'] = {'variable': 'H', 'value': field_strength, 'unit': 'A/m'}
        else:
            print('No Magnetic field strength defined!')

        mag_flux = []
        if len(material_dictionary['Magnetic polarization']['value']) == len(
                material_dictionary['Magnetic field strength']['value']):
            for i in range(len(material_dictionary['Magnetic field strength']['value'])):
                temp_list = [float(a + b) for a, b in
                             zip([j / 1000.0 for j in material_dictionary['Magnetic polarization']['value'][i]],
                                 [h * mu_0 for h in material_dictionary['Magnetic field strength']['value'][i]])]
                mag_flux.append(temp_list)

            material_dictionary['Magnetic flux density'] = {'variable': 'B', 'value': mag_flux, 'unit': 'T'}

        print('Specific total loss [W/kg] == ', sheet.iloc[1, 4])
        specific_loss = []
        if 'Specific total loss [W/kg]' == sheet.iloc[1, 4]:
            for i in range(len(indices_frequency) - 1):
                temp_list = [sheet.iloc[i, 4] for i in
                             range(2 + indices_frequency[i], 2 + indices_frequency[i + 1])]
                temp_list = [float(value) for value in temp_list if value != 0.0 and value != "" and value == value]
                temp_list.insert(0, 0.0)
                specific_loss.append(temp_list)

            material_dictionary['Specific total loss'] = {'variable': 'P', 'value': specific_loss, 'unit': 'W/kg'}
        else:
            print('No Specific total loss defined!')

        permeability = []
        if 'Relative permeability \n[-]' == sheet.iloc[1, 5]:
            for i in range(len(indices_frequency) - 1):
                temp_list = [sheet.iloc[i, 5] for i in
                             range(2 + indices_frequency[i], 2 + indices_frequency[i + 1])]
                temp_list = [float(value) for value in temp_list if value != 0.0 and value != "" and value == value]
                temp_list.insert(0, 0.0)
                permeability.append(temp_list)

            material_dictionary['Relative permeability'] = {'variable': 'mur', 'value': permeability, 'unit': '-'}
        else:
            print('No Relative permeability defined!')

        return material_dictionary

    @staticmethod
    def read_excel_10_bhcurve(sheet):
        """ Excel-file where the data for BH-Curve and losses are in the same tabular.
        In this case "BH-Curve" has to be in the cell (1,0) of the excel sheet. """

        material_dictionary = {}

        print('Frequency [Hz] == ', sheet.iloc[1, 1])
        frequency = []
        indices_frequency = []

        if 'Frequency [Hz]' == sheet.iloc[1, 1]:
            temp_list = [sheet.iloc[i, 1] for i in range(2, sheet.shape[0])]
            # temp_list = list(filter(None, temp_list))
            # frequency = np.array([float(i) for i in temp_list])
            counter = -1
            for item in temp_list:
                counter += 1
                if item is not None and item == item: # check if item is not None and not NaN
                    frequency.append(float(item))
                    indices_frequency.append(counter)

            indices_frequency.append(sheet.shape[0] - 2)  # work around since there is no frequency value in the last row
            if indices_frequency[0] != 0:
                raise Exception("Check if Frequency is defined for BHCurve.")

            material_dictionary['Frequency'] = {'variable': 'F', 'value': frequency[1:], 'unit': 'Hz'}
        else:
            print('No Frequency defined!')

        ''' Read BH-Curve '''
        polarization_bh = []
        if 'Magnetic polarization [mT]' == sheet.iloc[1, 2]:
            k = 0
            temp_list = [sheet.iloc[j, 2] for j in range(2 + indices_frequency[k], 2 + indices_frequency[k + 1])]
            temp_list = [float(value) for value in temp_list if value != 0.0 and value != "" and value == value]
            temp_list.insert(0, 0.0)
            polarization_bh.append(temp_list)

            material_dictionary['J Values'] = {'variable': 'J', 'value': polarization_bh, 'unit': 'mT'}
        else:
            print('No Magnetic polarization defined!')

        field_strength_bh = []
        if 'Magnetic field strength [A/m]' == sheet.iloc[1, 3]:
            k = 0
            temp_list = [sheet.iloc[j, 3] for j in range(2 + indices_frequency[k], 2 + indices_frequency[k + 1])]
            temp_list = [float(value) for value in temp_list if value != 0.0 and value != "" and value == value]
            temp_list.insert(0, 0.0)
            field_strength_bh.append(temp_list)

            material_dictionary['H Values'] = {'variable': 'H', 'value': field_strength_bh, 'unit': 'A/m'}
        else:
            print('No Magnetic field strength defined!')

        ''' Read Loss-Values '''
        specific_loss = []
        if 'Specific total loss [W/kg]' == sheet.iloc[1, 4]:
            for i in range(1, len(indices_frequency) - 1):
                temp_list = [sheet.iloc[j, 4] for j in range(2 + indices_frequency[i], 2 + indices_frequency[i + 1])]
                temp_list = [float(value) for value in temp_list if value != 0.0 and value != "" and value == value]
                temp_list.insert(0, 0.0)
                specific_loss.append(temp_list)

            material_dictionary['Specific total loss'] = {'variable': 'P', 'value': specific_loss, 'unit': 'W/kg'}
        else:
            print('No Specific total loss defined!')

        mag_flux = []
        if 'Magnetic flux density \n[T]' == sheet.iloc[1, 5]:
            for i in range(1, len(indices_frequency) - 1):
                temp_list = [sheet.iloc[j, 5] for j in range(2 + indices_frequency[i], 2 + indices_frequency[i + 1])]
                temp_list = [float(value) for value in temp_list if value != 0.0 and value != "" and value == value]
                temp_list.insert(0, 0.0)
                mag_flux.append(temp_list)

            material_dictionary['Magnetic flux density'] = {'variable': 'B', 'value': mag_flux, 'unit': 'T'}
        else:
            print('No Magnetic flux density defined!')

        return material_dictionary

    @staticmethod
    def read_excel_stressstrain(sheet):
        """ Excel-file where the data for the Stress_Strain curve are listed. """

        material_dictionary = {}

        print('Temperature [C] == ', sheet.iloc[1, 1])
        temperature = np.array([])
        indices_temperature = []
        if 'Temperature [C]' == sheet.iloc[1, 1]:
            temp_list = [sheet.iloc[i, 1] for i in range(2, sheet.shape[0])]
            # temp_list = list(filter(None, temp_list))
            # temperature = np.array([float(i) for i in temp_list])
            temp_temperature = []
            counter = -1
            for item in temp_list:
                counter += 1
                if item is not None and item == item: # check if item is not None and not NaN
                    temp_temperature.append(float(item))
                    indices_temperature.append(counter)

            indices_temperature.append(sheet.shape[0] - 2)  # work around since there is no temperature value in the last row
            if indices_temperature[0] != 0:
                raise Exception("Check if Temperature is defined for StressStrainCurve.")

            temperature = np.array(temp_temperature)
        #     material_dictionary['Frequency'] = {'variable': 'F', 'value': frequency[1:], 'unit': 'Hz'}
        else:
            print('No Temperature defined!')

        ''' Read StressStrain-Curve '''
        temp_stress = []
        if 'Stress [MPa]' == sheet.iloc[1, 2]:
            for k in range(len(temperature)):
                temp_list = [sheet.iloc[j, 2] for j in range(2 + indices_temperature[k], 2 + indices_temperature[k + 1])]
                temp_list = [float(value) for value in temp_list if value != ""]
                temp_stress.append(temp_list)

        #     polarization_bh = np.array(temp_polarization_bh)
        #     material_dictionary['J Values'] = {'variable': 'J', 'value': polarization_bh, 'unit': 'mT'}
        else:
            print('No Stress defined!')

        temp_plastic_strain = []
        if 'Plastic Strain [-]' == sheet.iloc[1, 3]:
            for k in range(len(temperature)):
                temp_list = [sheet.iloc[j, 3] for j in range(2 + indices_temperature[k], 2 + indices_temperature[k + 1])]
                temp_list = [float(value) for value in temp_list if value != ""]
                temp_plastic_strain.append(temp_list)

        #     field_strength_bh = np.array(temp_field_strength_bh)
        #     material_dictionary['H Values'] = {'variable': 'H', 'value': field_strength_bh, 'unit': 'A/m'}
        else:
            print('No Plastic Strain defined!')

        if len(temperature) > 0 and len(temp_stress) > 0 and len(temp_plastic_strain) > 0:
            material_dictionary['StressStrainCurve'] = {}
            for i in range(len(temperature)):
                current_temperature = temperature[i]
                material_dictionary['StressStrainCurve'][str(current_temperature)] = \
                    {'Stress': {'variable': 'Stress at ' + str(current_temperature) + ' C',
                                'value': temp_stress[i], 'unit': 'MPa'},
                     'PlasticStrain': {'variable': 'PlasticStrain at ' + str(current_temperature) + ' C',
                                       'value': temp_plastic_strain[i], 'unit': '-'}}

        return material_dictionary
