#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 7331 $
$LastChangedDate:: 2023-10-17 08:34:10 +0200#$

This file is part of SyMSpace.

Copyright (c) 2020, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
import os
from symspace.wrapper.symwrapper import SymField, SymProject, SymContainer
import uuid


class ComponentFilter:
    def __init__(self, comp):
        if isinstance(comp, str):
            if os.path.isfile(comp):
                self.prj = SymProject.load(comp)
            else:
                raise ValueError('File {} not found'.format(comp))
        elif isinstance(comp, SymProject):
            self.prj = comp
        else:
            raise ValueError('Undefined type of comp')

    def addProvides(self, type: str, display_name: str = None, guid: str = '') -> SymContainer:
        """Add provide filter

        Parameters
        ----------
        type: str
        display_name :str
        guid: str
            Unique identifier
        Returns
        -------
        rule : SymContainer
        """
        if not self.prj['Filter.provides']:
            provides = SymContainer('provides', 'Provides property')
            self.prj['Filter'].addParam(provides)
        rule = SymContainer(type, 'Provides ' + type)
        self.prj['Filter.provides'].addParam(rule)
        operator = SymField('operator', 'Operator')
        rule.addParam(operator)
        operator.setData('eq')

        provtype = SymField('type')
        rule.addParam(provtype)
        provtype.setData('PROPERTY')

        prop = SymField('property')
        rule.addParam(prop)
        prop.setData('type')

        value = SymField('value')
        rule.addParam(value)
        value.setData(type)

        # Set GUID
        unique_id = SymField('guid', 'Globally Unique Identifier')
        rule.addParam(unique_id)
        if guid == '':
            unique_id.setData(str(uuid.uuid4()))
        else:
            unique_id.setData(guid)

        disp = SymField('displayName')
        rule.addParam(disp)
        if display_name:
            disp.setData(display_name)
        else:
            disp.setData(type.replace('_', ' '))
        return rule

    def addRequiresType(self, type: str, display_name: str = None, optional: bool = False,
                        n_min: int = None, n_max: int = None) -> SymContainer:
        """

        Parameters
        ----------
        type: str
        display_name :str
        optional: bool
        n_min:int
        n_max:int

        Returns
        -------
        opand : SymContainer
        """
        if not self.prj['Filter.requires']:
            requires = SymContainer('requires', 'Requires property')
            self.prj['Filter'].addParam(requires)

        if not self.prj['Filter.requires.AND']:
            requires = SymContainer('AND')
            self.prj['Filter.requires'].addParam(requires)

        opand = SymContainer('AND_' + type, 'Requires ' + type)
        self.prj['Filter.requires.AND'].addParam(opand)
        rule = SymContainer(type, type)
        opand.addParam(rule)

        operator = SymField('operator', 'Operator')
        rule.addParam(operator)
        operator.setData('eq')

        provtype = SymField('type')
        rule.addParam(provtype)
        provtype.setData('PROPERTY')

        prop = SymField('property')
        rule.addParam(prop)
        prop.setData('provides_type')

        value = SymField('value')
        rule.addParam(value)
        value.setData(type)

        disp = SymField('displayName')
        opand.addParam(disp)
        if display_name:
            disp.setData(display_name)
        else:
            disp.setData(type.replace('_', ' '))

        if optional:
            optional_param = SymField('optional', 'Optional flag')
            opand.addParam(optional_param)
            optional_param.setData(True)

        if isinstance(n_min, int):
            min_param = SymField('min', 'Minimum amount')
            opand.addParam(min_param)
            min_param.setData(n_min)

        if isinstance(n_max, int):
            max_param = SymField('max', 'Maximum amount')
            opand.addParam(max_param)
            max_param.setData(n_max)

        # Set GUID
        guid = SymField('guid', 'Globally Unique Identifier')
        opand.addParam(guid)
        guid.setData(str(uuid.uuid4()))

        return opand

    def addNodePath(self, nodepath: str, display_name: str = None) -> SymContainer:
        """

        Parameters
        ----------
        nodepath : str
        display_name :str

        Returns
        -------
        opand : SymContainer
        """
        if not self.prj['Filter.requires']:
            requires = SymContainer('requires', 'Requires property')
            self.prj['Filter'].addParam(requires)

        if not self.prj['Filter.requires.AND']:
            requires = SymContainer('AND')
            self.prj['Filter.requires'].addParam(requires)

        name = nodepath.split('.')[-1]
        opand = SymContainer('AND_' + name, 'Requires ' + name)
        self.prj['Filter.requires.AND'].addParam(opand)
        rule = SymContainer(name, name)
        opand.addParam(rule)

        operator = SymField('operator', 'Operator')
        rule.addParam(operator)
        operator.setData('contains')

        provtype = SymField('type')
        rule.addParam(provtype)
        provtype.setData('PROPERTY')

        prop = SymField('property')
        rule.addParam(prop)
        prop.setData('nodePath')

        value = SymField('value')
        rule.addParam(value)
        value.setData(nodepath)

        disp = SymField('displayName')
        opand.addParam(disp)
        if display_name:
            disp.setData(display_name.replace('_', ' '))
        else:
            disp.setData(nodepath)

        # Set GUID
        guid = SymField('guid', 'Globally Unique Identifier')
        opand.addParam(guid)
        guid.setData(str(uuid.uuid4()))

        return opand

    def addGeometryProvides(self):
        cad = ['FreeCAD', 'Creo']
        properties = ['FEM_MAG2D',
                      'FEM_MAG3D',
                      'FEM_STRESS2D',
                      'FEM_THERM2D']
        if self.prj['Filter.provides']:
            for f in self.prj['Filter.provides'].getParam():
                if f.getName() in properties:
                    properties.remove(f.getName())

        for cadname in cad:
            for prop in properties:
                print(f'Add property {prop}')
                self.addProvides(prop).setName(prop + '_' + cadname)


if __name__ == '__main__':
    CurProject = locals()['CurProject']
    dqmodel = ComponentFilter(CurProject)
    dqmodel.addRequiresType('dqModel')
