# !/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 4091 $
$LastChangedDate:: 2021-02-09 18:29:30 +0100#$

This file is part of SyMSpace.

Copyright (c) 2020, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
import sys
import asyncio
import getpass
import platform
import websockets
import json_tricks
from symspace.osutils.processutils import needsPython


class HerokuClient:
    """Client class for Model-As-A-Service at Heroku.

    This is the client part of a client-server websocket application.
    - Opens a connection to server
    - sends data and user information as json string
    - retrieves server output as json string
    - converts json string to python objects
    """

    def __init__(self, server_uri: str, server_port: int):
        """
        Heroku-Client which will send and receive data from the Heroku-Server

        Parameters
        ----------
        server_uri: str
            URI of the server e.g. "symspace-demo-app10.herokuapp.com"
            For debugging with a local server use "localhost"
        server_port: int
            Listening port of the server

        """
        self.server_uri = server_uri
        self.server_port = int(server_port)

    @staticmethod
    async def client(input_data_json: str, user_data_json: str, server_uri: str, server_port: int) -> tuple:
        """
        Asynchronous client method

        Parameters
        ----------
        input_data_json : str
            JSON Serialized input data
        user_data_json: str
            JSON Serialized user/client-system data
        server_uri: str
            URI of the server
        server_port: int
            Listening port of the server

        Returns
        -------
        tuple
            output_data_json: str
                JSON Serialized output data
            log: str
                log messages
        """
        websocket_resource_url = 'ws://{}:{}'.format(server_uri, server_port)

        '''The (asynchronous) connection is closed when exiting the context'''
        async with websockets.connect(websocket_resource_url) as websocket:
            await websocket.send(user_data_json)
            await websocket.send(input_data_json)

            output_data_json = await websocket.recv()
            log = await websocket.recv()
            return output_data_json, log

    def send_data(self, input_data):
        """
        Send data to the server

        Parameters
        ----------
        input_data
            Data to be processed by the server

        Returns
        -------
        output_data
            Data returned by the server
        """
        input_data_json = json_tricks.dumps(input_data)

        output_data_json = self.send_json_data(input_data_json)

        '''Transform json string back to dict of numpy arrays, etc..'''
        output_data = json_tricks.loads(output_data_json)
        return output_data

    @needsPython
    def send_json_data(self, input_data_json: str):
        """
        Send data to the server

        Parameters
        ----------
        input_data_json: str
            JSON Serialized data to be processed by the server

        Returns
        -------
        output_data_json: str
            JSON Serialized data returned by the server
        """
        user_data_json = json_tricks.dumps({
            'sysinfo': platform.uname()._asdict(),
            'username': getpass.getuser()
        })

        start_client = HerokuClient.client(input_data_json, user_data_json, self.server_uri, self.server_port)

        if sys.version_info >= (3, 7):
            '''Needs Python >= 3.7'''
            output_data_json, log = asyncio.run(start_client)
        else:
            '''For Python < 3.7'''
            output_data_json, log = asyncio.get_event_loop().run_until_complete(start_client)

        '''Transform json string back to dict of numpy arrays, etc..'''
        return output_data_json
